package com.accuver.cem.view.frag;


import com.accuver.jpcem.CEMApplication;
import com.accuver.jpcem.R;

import android.annotation.TargetApi;
import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.app.Activity;
import android.app.FragmentTransaction;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.app.FragmentActivity;
import android.support.v4.view.ViewPager;
import android.util.Log;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;


@TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH) 
public class Frag_TabMain extends FragmentActivity implements ActionBar.TabListener {

	private static final String TAG = Frag_TabMain.class.getSimpleName();
	
	public static final int RESULT_ID_CLOSEST = 10000;
	
	
	public static final String Action_FirstShowFragIndex = "FirstShowFragIndex";
	private ViewPager viewPager;
	private Frag_TabAdapter mAdapter;
	private ActionBar actionBar;
	// Tab titles
	private int[] tabs_mode_engineer = { R.string.main_menu_title_1, R.string.main_menu_title_2, R.string.main_menu_title_3};
	private int[] tabs_mode_normal = { R.string.main_menu_title_1, R.string.main_menu_title_2};
	private int[] tabs;
	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.frag_main);

		// Initilization
		Log.d(TAG, "onCreate() - start");
		viewPager = (ViewPager) findViewById(R.id.pager);
		actionBar = getActionBar();
		mAdapter = new Frag_TabAdapter(getSupportFragmentManager());
		
		viewPager.setAdapter(mAdapter);
		actionBar.setHomeButtonEnabled(false);
		actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);

		actionBar.setDisplayShowTitleEnabled(false);
		actionBar.setDisplayShowHomeEnabled(false);
		
		// set Menu
		tabs = tabs_mode_engineer.clone();
		
		// Adding Tabs
		for (int i=0; i<tabs.length; i++) {
			actionBar.addTab(actionBar.newTab().setText(tabs[i]).setTabListener(this));
		}

		Intent intent = getIntent();
		if (intent != null) {
			int index = intent.getIntExtra(Action_FirstShowFragIndex, -1);
			
			if (index > -1 /* && csetup.isServiceMode == false*/) {
				switch (index) {
				case 1:
					viewPager.setCurrentItem(0, true);
					actionBar.setSelectedNavigationItem(0);
					break;
				case 2:
					viewPager.setCurrentItem(1, true);
					actionBar.setSelectedNavigationItem(1);
					break;
				case 3:
					viewPager.setCurrentItem(2, true);
					actionBar.setSelectedNavigationItem(2);
					break;
				}
			}
		}
		
		/**
		 * on swiping the viewpager make respective tab selected
		 * */
		viewPager.setOnPageChangeListener(new ViewPager.OnPageChangeListener() {
			@Override
			public void onPageSelected(int position) {
				Log.d(TAG,"onPageSelected");
				actionBar.setSelectedNavigationItem(position);
			}

			@Override
			public void onPageScrolled(int arg0, float arg1, int arg2) {
				//Log.d(TAG,"onPageScrolled");
				hideSoftKeyboard();
			}

			@Override
			public void onPageScrollStateChanged(int state) {
				//nothing
			}
		});
		
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data){
		if(requestCode == RESULT_ID_CLOSEST){
			if(resultCode == Activity.RESULT_OK){
				Log.e(TAG,"closest test ok");
				mAdapter.goTest();
			}
		}
	}

	
	@Override
	public void onTabReselected(Tab tab, FragmentTransaction ft) {
		Log.d(TAG,"onTabReselected");
	}

	@Override
	public void onTabSelected(Tab tab, FragmentTransaction ft) {
		int selected = tab.getPosition();
		viewPager.setCurrentItem(selected);
		Log.d(TAG,"onTabSelected index:"+selected);
		hideSoftKeyboard();
		if(selected == 1){ //history
			mAdapter.updateHistory();
		}
		else if(selected == 2){ //normal setting
			mAdapter.updatePreference();
		}
	}

	@Override
	public void onTabUnselected(Tab tab, FragmentTransaction ft) {
		Log.d(TAG,"onTabUnselected");
	}
	
	@Override
	protected void onResume() {
		Log.d(TAG, "onResume() - start");
		if (getIntent().getExtras() != null)
		{
			Bundle bundle = (Bundle) getIntent().getExtras();
			int tsetStart = bundle.getInt("TsetStart", 0);
			
			switch (tsetStart) {
			case 1:
				viewPager.setCurrentItem(0, true);
				actionBar.setSelectedNavigationItem(0);
				break;
			case 2:
				viewPager.setCurrentItem(1, true);
				actionBar.setSelectedNavigationItem(1);
				break;
			case 3:
				viewPager.setCurrentItem(2, true);
				actionBar.setSelectedNavigationItem(2);
				break;
			}
		}
		
		super.onResume();
		
		
		CEMApplication app = (CEMApplication)getApplication();
		app.setActivityHandler(mHandler);
	}
	
	private Handler mHandler = new Handler(){
		@Override
		public void handleMessage(Message msg){
			if(msg.what == CEMApplication.MSG_BGTEST_START){
				viewPager.setCurrentItem(0, true);
				actionBar.setSelectedNavigationItem(0);
				mAdapter.runBGTest();
			}
		}
	};
	
	@Override
	protected void onPause() {
		super.onPause();
		
		CEMApplication app = (CEMApplication)getApplication();
		app.setActivityHandler(null);
	}
	
	@Override
	public void onDestroy()
	{
		Log.d(TAG, "onDestroy() - start"); 
		if (getIntent().getExtras() != null)
		{
			Bundle bundle = (Bundle) getIntent().getExtras();
			int tsetStart = bundle.getInt("TsetStart", 0);
			if(tsetStart != 0){
//				MainActivity.MAIN_ACTVITY.finish();
			}
		}
		super.onDestroy();
		
        SharedPreferences pref = getSharedPreferences("MEASURE_PREF", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();
        editor.clear();
        editor.commit();
	}
	
	
	public void hideSoftKeyboard() {
	    if(getCurrentFocus()!=null) {
	        InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
	        inputMethodManager.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), 0);
	    }
	}
	
	public void hideSoftKeyboard(EditText et) {
		if(et == null) {
			hideSoftKeyboard();
			return;
		}
		
        InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        inputMethodManager.hideSoftInputFromWindow(et.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
	}
}
