package com.accuver.cem.view.graph;

import android.content.Context;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.WindowManager;

public class SpeedChartView extends SurfaceView implements SurfaceHolder.Callback{

	private final String TAG = "SpeedChartView";
	private SpeedMeterThread_Vertical mSpeedMeterThread;
	private int mWidthMeasureSpec, mHeightMeasureSpec;
	private Context mContext;
	
	public SpeedChartView(Context context) {
		super(context);
		init(context);
		mContext = context;
	}
	
	public SpeedChartView(Context context, AttributeSet attr){
		super(context, attr);
		init(context);
		mContext = context;
	}
	
	private void init(Context context) {
		WindowManager wm = (WindowManager)context.getSystemService(Context.WINDOW_SERVICE);
		int width = wm.getDefaultDisplay().getWidth();
		int height = wm.getDefaultDisplay().getHeight();
		Log.e(TAG,"[onInit] width:"+width+"/height:"+height);
		height = height / 3;
		SurfaceHolder holder = getHolder();
		holder.addCallback(this);
		setFocusable(true);
	}
	
	public void pause(){
		if(mSpeedMeterThread != null){
			mSpeedMeterThread.pauseThread();
		}
	}
	public void resume(){
		if(mSpeedMeterThread != null){
			mSpeedMeterThread.resumeThread();
		}
	}
	public void setNeedDisplay(){
		if(mSpeedMeterThread != null){
			mSpeedMeterThread.setNeedDisplay();
		}
	}
	public void setActivation(boolean isActive){
		if(mSpeedMeterThread != null){
			mSpeedMeterThread.setActive(isActive);
		}
	}
	
	public void stop(){
		if(mSpeedMeterThread != null){
			//mPunchThread.interrupt();
			boolean retry = true;
			mSpeedMeterThread.setRunnable(false);
			while (retry) {
                try {
                	mSpeedMeterThread.join();
                    retry = false;
                } catch (InterruptedException e) {
                }
            }
			mSpeedMeterThread = null;
		}
	}
	
	public void setMaxThroughput(int maxkbps, int delimeterCount, String unit){
		if(mSpeedMeterThread != null){
			mSpeedMeterThread.setMaxKbps(maxkbps, delimeterCount, unit);
		}		
	}
	
	public void setAverageThroughput(double kbps){
		if(mSpeedMeterThread != null){
			mSpeedMeterThread.setAverageThroughput(kbps);
		}
			
	}
	public void setInstantThroughput(double kbps){
		if(mSpeedMeterThread != null){
			mSpeedMeterThread.setInstantThroughput(kbps);
		}
	}
	
	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		// TODO Auto-generated method stub
		super.onMeasure(widthMeasureSpec, heightMeasureSpec);
		if(mSpeedMeterThread == null){
			mWidthMeasureSpec = MeasureSpec.getSize(widthMeasureSpec);
			mHeightMeasureSpec = MeasureSpec.getSize(heightMeasureSpec);
			Log.e(TAG,"[onMeasure] width:"+mWidthMeasureSpec+"/height:"+mHeightMeasureSpec);
		}
	}
	
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
		// TODO Auto-generated method stub
		Log.e(TAG,"[surfaceChanged] width:"+width+"/height:"+height);
		if(mSpeedMeterThread == null){
			mWidthMeasureSpec = MeasureSpec.getSize(width);
			mHeightMeasureSpec = MeasureSpec.getSize(height);
			mSpeedMeterThread = new SpeedMeterThread_Vertical(holder, mContext);
			mSpeedMeterThread.setDisplaySize(mWidthMeasureSpec, mHeightMeasureSpec);
			mSpeedMeterThread.setRunnable(true);
			mSpeedMeterThread.start();
		}
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		// TODO Auto-generated method stub
		Log.e(TAG,"[surfaceCreated] width:"+mWidthMeasureSpec+"/height:"+mHeightMeasureSpec);
//		mSpeedMeterThread = new SpeedMeterThread(holder, mContext);
//		mSpeedMeterThread.setDisplaySize(mWidthMeasureSpec, mHeightMeasureSpec);
//		mSpeedMeterThread.setRunnable(true);
//		mSpeedMeterThread.start();
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		// TODO Auto-generated method stub
		stop();
	}
	public Handler mMessageHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case 11:
				break;
			case 12:
				break;
			case 13:
				break;
			}
		}
	};
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		// TODO Auto-generated method stub
		//Log.e("birdea","touched:"+event.getAction());
		//int ex = (int)event.getX();
		//int ey = (int)event.getY();
		//mSpeedMeterThread.setTouchPosition(ex, ey);
		switch(event.getAction()){
		case MotionEvent.ACTION_UP:
			break;
		}
		return true;
	}

}
