package com.accuver.cem.view.graph;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.RadialGradient;
import android.graphics.RectF;
import android.util.Log;
import android.view.SurfaceHolder;

public class SpeedMeterThread_Vertical extends Thread {

	private final static String TAG = SpeedMeterThread_Vertical.class.getSimpleName();
	private final static int DELAY = 5;
	private int view_width, view_height, center_x, center_y;
	private int cRadius;
	private int xPos, yPos;
	private double xAvgRate, xAvgRate_ed, xInstRate, xInstRate_ed;
	private double xInstValue;
	private double mMaxKbps, mDegreeGap;
	private float mScale = 0;
	private String mUnit = "";
	private SurfaceHolder sHolder = null;
	private volatile boolean isRunnable;
	private static final int COLOR_BACKGROUND_TOP = Color.rgb(0, 154, 233);
	private static final int COLOR_BACKGROUND_BOTTOM = Color.rgb(153, 216, 255);
	private static final int COLOR_BACKGROUND_CENTER = Color.rgb(42, 103, 192);
	private static final int COLOR_BACKGROUND_SIDE = Color.rgb(0, 154, 224);
	private static final int COLOR_INNER_ARC_1 = Color.rgb(255, 99, 0);
	private static final int COLOR_INNER_ARC_2 = Color.rgb(245, 245, 245);
	private static final int COLOR_INNER_ARC_3 = Color.rgb(154, 154, 154);
	private static final int COLOR_PIN_INSTANT = Color.rgb(255, 99, 0);
	private static final int COLOR_PIN_AVERAGE_1 = Color.rgb(255, 186, 46);
	private static final int COLOR_PIN_AVERAGE_2 = Color.rgb(255, 211, 122);
	private static final int COLOR_MARK_AVERAGE = Color.rgb(224, 150, 0);
	private static final int COLOR_OUTER_ARC_1 = Color.rgb(158, 190, 255);
	private static final int COLOR_OUTER_ARC_2 = Color.rgb(219, 231, 255);
	private static final int COLOR_OUTER_ARC_3 = Color.rgb(245, 245, 245);
	private static final int COLOR_TEXT_MARK = Color.rgb(255, 255, 255);

	public SpeedMeterThread_Vertical(SurfaceHolder holder, Context context) {
		sHolder = holder;
		init(context);
		setName("SpeedMeter");
	}

	private void init(Context context) {
		xPos = 0;
		yPos = 0;
		cRadius = 100;
		mMaxKbps = 100 * 1000;
		mDegreeGap = 5;
		xAvgRate = 0;
		xInstRate = 0;
		mScale = context.getResources().getDisplayMetrics().density;
	}

	public void setMaxKbps(int maxkbps, int delimeterCount, String unit) {
		mMaxKbps = maxkbps;
		mDegreeGap = delimeterCount;
		mUnit = unit;
	}

	public void setDisplaySize(int width, int height) {
		view_width = width;
		view_height = height;
		center_x = width / 2;
		center_y = height / 2;
	}

	public void setRunnable(boolean runable) {
		isRunnable = runable;
	}

	@Override
	public void run() {
		Canvas canvas = null;
		Paint paint = new Paint();
		RectF rectf = new RectF();
		setNeedDisplay();
		while (isRunnable) {
			if(isPause)
			{
				try {
					sleep(50);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				continue;
			}
			
			if(bInvalidate || bActive){
				try {
					canvas = sHolder.lockCanvas(null);
					synchronized (sHolder) {
						if (canvas == null)
							continue;
						// /////////////////////////////////////////////
						// canvas.drawColor(COLOR_BACKGROUND);
						draw_GradientBackGround(canvas, paint);
						// draw_Text(canvas, paint);
						draw_Arc_SpeenGage(canvas, paint, rectf);
						draw_Arc_Delimeters(canvas, paint, rectf);
						draw_CenterCircle(canvas, paint, rectf);
						draw_TextEachValue(canvas, paint);
						// draw_Arc_SpeedText(canvas, paint);
						// draw_Line_SpeedGage(canvas, paint);
						// draw_CurruntTime(canvas, paint);
						// draw_CenterPin(canvas, paint);
						// draw_CenterBitmapPin(canvas);
						// /////////////////////////////////////////////
					}
				} catch (Exception e) {
					e.printStackTrace();
				} finally {
					if (canvas != null) {
						sHolder.unlockCanvasAndPost(canvas);
					}
				}
				
				try {
					sleep(DELAY);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				bInvalidate = false;
			}
			else {
				try {
					sleep(50);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	}

	private void draw_GradientBackGround(Canvas canvas, Paint paint) {
		// start at 0,0 and go to 0,max to use a vertical
		// gradient the full height of the screen.
		RadialGradient gradient = new android.graphics.RadialGradient(view_width, center_y, view_width, COLOR_BACKGROUND_CENTER, COLOR_BACKGROUND_SIDE, android.graphics.Shader.TileMode.CLAMP);
		//paint.setShader(new LinearGradient(0, 0, 0, view_height, COLOR_BACKGROUND_TOP, COLOR_BACKGROUND_BOTTOM, Shader.TileMode.MIRROR));
		paint.setShader(gradient);
		// Create a radial gradient
		canvas.drawPaint(paint);
		paint.setShader(null);
	}

	private void check_MaxValue(double value){
		if(value > mMaxKbps){
			mMaxKbps = value * (double)1.2;
		}
	}

	public void setAverageThroughput(double kbps) {
		check_MaxValue(kbps);
		xAvgRate_ed = xAvgRate;
		xAvgRate = (double) kbps / ((double) mMaxKbps);
		if (xAvgRate > 1) {
			xAvgRate = 1;
		}
	}

	public void setInstantThroughput(double kbps) {
		check_MaxValue(kbps);
		xInstValue = kbps;
		xInstRate_ed = xInstRate;
		xInstRate = (double) kbps / ((double) mMaxKbps);
		if (xInstRate > 1) {
			xInstRate = 1;
		}
	}

	private void draw_Arc_Delimeters(Canvas canvas, Paint paint, RectF rectf) {
		// Common Set
		paint.setAntiAlias(true);
		double x1 = 0f, y1 = 0f, x2 = 0f, y2 = 0f;
		double r1 = 0f, r2 = 0f;
		r1 = view_width / 10 * 9 - 20*mScale;
		r2 = view_width / 10 * 9 - 35*mScale;
		for (int degree = 90; degree <= 270; degree += 5) {
			x1 = (view_width + r1 * Math.cos(Math.toRadians(degree)));
			y1 = (center_y + r1 * Math.sin(Math.toRadians(degree)));
			x2 = (view_width + r2 * Math.cos(Math.toRadians(degree)));
			y2 = (center_y + r2 * Math.sin(Math.toRadians(degree)));
			if (isInnerDegree(degree)) {
				paint.setColor(COLOR_MARK_AVERAGE);
				paint.setStrokeWidth(2*mScale);
			} else {
				paint.setColor(COLOR_OUTER_ARC_1);
				paint.setStrokeWidth(2*mScale);
			}
			// Log.e(TAG,"draw_Arc_Delimeters - degree:"+degree+"/x1:"+x1+"/y1:"+y1+"/x2:"+x2+"/y2:"+y2);
			// Log.e(TAG,"draw_Arc_Delimeters - degree:"+degree+"/toRadians:"+Math.toRadians(degree));
			canvas.drawLine((float) x1, (float) y1, (float) x2, (float) y2, paint);
		}
	}

	private boolean isInnerDegree(int degree) {
		double deg = (degree - 90);
		if (deg == 0) {
			return true;
		} else {
			deg = deg / 180;
			if (deg < xAvgRate_ed) {
				return true;
			}
		}
		return false;
	}

	private void draw_Arc_SpeedText(Canvas canvas, Paint paint) {
		paint.setAntiAlias(true);
		paint.setTextSize(15);
		paint.setStrokeWidth(5);
		paint.setStyle(Style.FILL);
		paint.setColor(Color.argb(255, 20, 122, 125));
		int radius = view_height / 10 * 2;
		canvas.drawText(String.format("%.2f", xInstValue / 1000), center_x - radius + 10, view_height - 2, paint);
	}

	private double getSpeedGage_Gap(int type) {
		double gap = 0;
		switch (type) {
		case 1: // to xAvgRate
			gap = (xAvgRate + xAvgRate_ed * 15) / 16;
			xAvgRate_ed = gap;
			break;
		case 2: // to xInstRate
			gap = (xInstRate + xInstRate_ed * 15) / 16;
			xInstRate_ed = gap;
			break;
		}
		return gap;
	}

	private void draw_Arc_SpeenGage(Canvas canvas, Paint paint, RectF rectf) {
		// Common Set
		int radius = 0;
		paint.setAntiAlias(true);

		// Outer Grey Arc.1
		paint.setStyle(Style.STROKE);
		paint.setStrokeWidth(10*mScale);
		paint.setColor(COLOR_OUTER_ARC_3);
		radius = view_width / 10 * 9;
		rectf.set(view_width - radius, center_y - radius, view_width + radius, center_y + radius);
		canvas.drawArc(rectf, 90, 270, true, paint);

		float avg = (float) (180 * getSpeedGage_Gap(1));
		// Outer Grey Arc.2-1
		paint.setStyle(Style.STROKE);
		paint.setStrokeWidth(2*mScale);
		paint.setColor(COLOR_OUTER_ARC_2);
		radius = view_width / 10 * 9 - (int)(20*mScale);
		rectf.set(view_width - radius, center_y - radius, view_width + radius, center_y + radius);
		canvas.drawArc(rectf, 90 + avg, 270 - avg, true, paint);

		// Arc ( Indicate for Average Throughput - 1 )
		paint.setStyle(Style.FILL_AND_STROKE);
		paint.setStrokeWidth(5*mScale);
		paint.setColor(COLOR_PIN_AVERAGE_1);
		radius = view_width / 10 * 9 - (int)(20*mScale);
		rectf.set(view_width - radius, center_y - radius, view_width + radius, center_y + radius);
		canvas.drawArc(rectf, 90, avg, true, paint);

		// Arc ( Indicate for Average Throughput - 2 )
		paint.setStyle(Style.STROKE);
		paint.setStrokeWidth(5*mScale);
		paint.setColor(COLOR_PIN_AVERAGE_2);
		radius = view_width / 10 * 9 - (int)(40*mScale);
		rectf.set(view_width - radius, center_y - radius, view_width + radius, center_y + radius);
		canvas.drawArc(rectf, 90, avg, true, paint);

		// Arc ( Indicate for Instant Throughput )
		paint.setStyle(Style.STROKE);
		paint.setStrokeWidth(5*mScale);
		paint.setColor(COLOR_PIN_INSTANT);
		radius = view_width / 10 * 9 - (int)(10*mScale);
		rectf.set(view_width - radius, center_y - radius, view_width + radius, center_y + radius);
		canvas.drawArc(rectf, 90, (float) (180 * getSpeedGage_Gap(2)), true, paint);
	}

	private void draw_TextEachValue(Canvas canvas, Paint paint) {
		float textSize = 18*mScale;
		paint.setAntiAlias(true);
		paint.setTextSize(textSize);
		paint.setStrokeWidth(0.5f*mScale);
		paint.setStyle(Style.FILL_AND_STROKE);
		paint.setColor(COLOR_TEXT_MARK);
		double x1 = 0f, y1 = 0f;
		double r1 = view_width / 10 * 6;
		String v = null;
		int v_cnt = 0;
		for (int degree = 90; degree <= 270; degree += 30) {
			x1 = ((view_width - textSize/2) + r1 * Math.cos(Math.toRadians(degree)));
			y1 = (center_y + r1 * Math.sin(Math.toRadians(degree)));
			v = "" + (int)((degree - 90) * mMaxKbps / (1000 * 180));
			canvas.drawText(String.format("%s", v), (float) x1, (float) y1 + 15, paint);
		}
	}

	private void draw_CenterCircle(Canvas canvas, Paint paint, RectF rectf) {
		// Common Set
		int radius = 0;
		paint.setAntiAlias(true);

		// Inner Arc.1
		paint.setColor(COLOR_INNER_ARC_1);
		paint.setStrokeWidth(1);
		paint.setStyle(Style.FILL);
		radius = view_width / 10 * 2;
		rectf.set(view_width - radius, center_y - radius, view_width + radius, center_y + radius);
		canvas.drawArc(rectf, 90, 270, true, paint);

		// Inner Arc.2
		paint.setColor(COLOR_INNER_ARC_2);
		paint.setStrokeWidth(1);
		paint.setStyle(Style.FILL);
		radius = view_width / 10 * 2 - (int)(5*mScale);
		rectf.set(view_width - radius, center_y - radius, view_width + radius, center_y + radius);
		canvas.drawArc(rectf, 90, 270, true, paint);

		// Inner Arc.3
		paint.setColor(COLOR_INNER_ARC_3);
		paint.setStrokeWidth(1);
		paint.setStyle(Style.FILL);
		radius = 5;
		rectf.set(view_width - radius, center_y - radius, view_width + radius, center_y + radius);
		canvas.drawArc(rectf, 90, 270, true, paint);
	}

	private boolean isPause = false;
	public void pauseThread() {
		isPause = true;
	}

	public void resumeThread() {
		isPause = false;
	}
	private boolean bInvalidate = false;
	public void setNeedDisplay() {
		bInvalidate = true;
	}
	private boolean bActive = false;
	public void setActive(boolean active){
		bActive = active;
	}
}
