package com.accuver.jpcem;


import com.accuver.cem.view.frag.Frag_TabMain;
import com.accuver.jpcem.R;

import com.samknows.measurement.MainService;
import com.samknows.ska.activity.SKAActivationActivity;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager.NameNotFoundException;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.LinearLayout;
import android.widget.TextView;



public class MainActivity extends Activity
{
	private static final String TAG = MainActivity.class.getSimpleName();
	private static final int RESULT_ID_START = 1;

	private int tsetStart;
	private TextView tv_current_version;
	private Handler mHandler = new Handler(){
		@Override
		public void handleMessage(Message msg){
			if(msg.what == CEMApplication.MSG_BGTEST_START){
				goMeasureScreen();
			}
		}
	};
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.activity_main);
		findView();
		
		if(!MainService.isExecuting()) {
			startActivityForResult(new Intent(this, SKAActivationActivity.class), RESULT_ID_START);
		}
		else {
			goMeasureScreen();
		}
	}

	
	@Override
	protected void onDestroy() {
		super.onDestroy();
	}

	@Override
	protected void onPause() {
		super.onPause();
		
		CEMApplication app = (CEMApplication)getApplication();
		app.setActivityHandler(null);
	}

	@Override
	protected void onResume()
	{
		super.onResume();
		
		CEMApplication app = (CEMApplication)getApplication();
		app.setActivityHandler(mHandler);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data){
		if(requestCode == RESULT_ID_START){
			if(resultCode == Activity.RESULT_OK){
				Log.d(TAG,"RESULT_ID_START ok");
				// 1. Check GPS ON
				LocationManager location = (LocationManager)this.getSystemService(Context.LOCATION_SERVICE);
				if(location.isProviderEnabled(LocationManager.GPS_PROVIDER) == false){
					showGpsSettingsAlert();	
				}
			}
			else if(resultCode == Activity.RESULT_CANCELED){
				// Activation fail, so finish
				MainActivity.this.finish();
			}
		}
	}
	
	private void findView()
	{		
		((LinearLayout) findViewById(R.id.ibtn_measurement)).setOnClickListener(mClickListener);
		((LinearLayout) findViewById(R.id.ibtn_setting)).setOnClickListener(mClickListener);
		((LinearLayout) findViewById(R.id.ibtn_history)).setOnClickListener(mClickListener);
		
		findViewById(R.id.ibtn_setting).setVisibility(View.VISIBLE);
		
		tv_current_version = (TextView)findViewById(R.id.aatv_app_version);

		try {
			String version_str = getResources().getString(R.string.main_current_version);
			String version_value = getPackageManager().getPackageInfo(getPackageName(), 0).versionName;
			tv_current_version.setText(""+version_str+" : "+version_value);
		} catch (NameNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	View.OnClickListener mClickListener = new View.OnClickListener()
	{
		@Override
		public void onClick(View view)
		{
			int id = view.getId();
			
			switch (id)
			{
			case R.id.ibtn_measurement:
			case R.id.ibtn_history:
			case R.id.ibtn_setting:
				doNextAction(id);
				break;
				
			
//				//Wakeup after 3min  
//				SK2AppSettings.getSK2AppSettingsInstance().background_test = true;
//				OtherUtils.rescheduleWakeup(MainActivity.this, 60 * 1000 );
//				break;

			}
		}
	};


	private void doNextAction(int id)
	{	
		String tExtra = Frag_TabMain.Action_FirstShowFragIndex;
		int tExtraValue = -1;
		
		switch (id)
		{
		case R.id.ibtn_measurement:
			tExtraValue = 1;
			break;
			
		case R.id.ibtn_history:
			tExtraValue = 2;
			break;
			
		case R.id.ibtn_setting:
			tExtraValue = 3;
			break;
		}
		
		Intent intent = new Intent(this, Frag_TabMain.class);		
		intent.putExtra(tExtra, tExtraValue);
		intent.putExtra("TsetStart", tsetStart);
		
		if (id > 0)
			startActivity(intent);
	}
	
	@Override
	public void onBackPressed()
	{
		String msg = getString(R.string.main_sure_to_exit);
		
		AlertDialog.Builder alertDialog = new AlertDialog.Builder(MainActivity.this);
		alertDialog.setPositiveButton(R.string.main_sure_to_exit_ok, new DialogInterface.OnClickListener()
		{
			@Override
			public void onClick(DialogInterface dialog, int which)
			{
				finish();
			}
		});
		alertDialog.setNegativeButton(R.string.main_sure_to_exit_cancel,
				new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
			}
		});
		alertDialog.setMessage(msg);
		alertDialog.setTitle("CEM");
		alertDialog.show();
	}
	
    private void showGpsSettingsAlert(){
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(this);
      
        alertDialog.setTitle("Information");
        alertDialog.setMessage(R.string.main_gps_alert_msg);
        alertDialog.setPositiveButton(R.string.main_gps_alert_settings, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog,int which) {
                Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                startActivity(intent);
            }
        });
        alertDialog.setNegativeButton(R.string.main_gps_alert_cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
            //dialog.cancel();
            }
        });
        alertDialog.show();
    }
    
    public void goMeasureScreen(){
    	//background testing!
		doNextAction(R.id.ibtn_measurement);
    }
}
