package com.samknows.measurement;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import org.json.JSONException;
import org.json.JSONObject;


import com.samknows.libcore.R;
import com.samknows.libcore.SKLogger;
import com.samknows.measurement.net.SubmitTestResultsAction;
import com.samknows.measurement.schedule.ScheduleConfig;
import com.samknows.measurement.schedule.TestDescription;
import com.samknows.measurement.schedule.datacollection.BaseDataCollector;
import com.samknows.measurement.storage.DBHelper;
import com.samknows.measurement.storage.ExportFile;
import com.samknows.measurement.storage.TestBatch;
import com.samknows.measurement.storage.TestResult;
import com.samknows.measurement.test.TestContext;
import com.samknows.measurement.test.TestExecutor;
import com.samknows.measurement.util.CustomTimestamp;
import com.samknows.tests.Param;
import com.samknows.tests.TestFactory;

import android.content.Context;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.telephony.TelephonyManager;
import android.util.Log;

/*  
 * This class is used to run the the tests when they are executed manually
 * implements a runnable interface and and uses an Handler in order to publish
 * the tests results to the interface
 */
public class ManualTest implements Runnable {
	
	private static final String TAG = ManualTest.class.getSimpleName();
	
	private Handler mHandler;
	private List<TestDescription> mTestDescription;
	private Context ctx;
	private AtomicBoolean run = new AtomicBoolean(true);
	public static boolean isExecuting = false;
	private boolean isRepeatTest = false;

	ManualTest(Context ctx, Handler handler, List<TestDescription> td) {
		mHandler = handler;
		mTestDescription = td;
		this.ctx = ctx;
	}


	/*
	 * Returns a ManualTest object if the manual_tests list of the schedule
	 * config is not empty and the MainService is not executing
	 */
	public static ManualTest create(Context ctx, Handler handler, StringBuilder RErrorDescription) {
		Storage storage = CachingStorage.getInstance();
		ScheduleConfig config = storage.loadScheduleConfig();
		if (config == null) {
			RErrorDescription.append(ctx.getString(R.string.manual_test_create_failed_1));
			SKLogger.e( ManualTest.class, RErrorDescription.toString());
			return null;
		}
		if (config.tests.size() == 0) {
			RErrorDescription.append(ctx.getString(R.string.manual_test_create_failed_2));
			SKLogger.e( ManualTest.class, RErrorDescription.toString());
			return null;
		}
//		if (MainService.isExecuting()) {
//			RErrorDescription.append(ctx.getString(R.string.manual_test_create_failed_3));
//			SKLogger.e(ManualTest.class, RErrorDescription.toString());
//			return null;
//		}
		/*
	   <string name="manual_test_create_failed_1">ManualTest cannot be initialized because storage does not contain the ScheduleConfig</string>
	   <string name="manual_test_create_failed_1">ManualTest cannot be initialized because storage does not contain the ScheduleConfig</string>
	   <string name="manual_test_create_failed_1">ManualTest cannot be initialized because storage does not contain the ScheduleConfig</string>
		*/
		
		return new ManualTest(ctx, handler, config.tests);
	}

	// returns the maximum amount of bytes used by the manual test
	public long getNetUsage() {
		return 0;
	}
	

	/*
	 * Runs all the test in manual tests
	 */

	@Override
	public void run() {
		DBHelper db = new DBHelper(ctx);
	
		
		sSetIsExecuting(true);
		
		// Start collectors for the passive metrics
		// Start tests
		long startTime = System.currentTimeMillis();
		JSONObject batch = new JSONObject();
		TestContext tc = TestContext.createManualTestContext(ctx);
		TestExecutor manualTestExecutor = new TestExecutor(tc);
		List<JSONObject> testsResults = new ArrayList<JSONObject>();
		List<JSONObject> passiveMetrics = new ArrayList<JSONObject>();
		manualTestExecutor.startInBackGround();
		Message msg;
		
		// make logging file name
		// 1 Time
		String time = new CustomTimestamp(){}.toString();
		time = time.substring(0, 15);
		
		TelephonyManager manager = (TelephonyManager)ctx.getSystemService(Context.TELEPHONY_SERVICE);
		String imsi = manager.getSubscriberId();
		if(imsi == null){
			WifiManager wifi = (WifiManager)ctx.getSystemService(Context.WIFI_SERVICE);
			WifiInfo info = wifi.getConnectionInfo();
			imsi = info.getMacAddress();
			imsi = imsi.replaceAll(":", "");
		}
		
		String model = Build.MODEL;
		String loggingFileName = ""+time+"_"+imsi+"_"+model+".csv";
		manualTestExecutor.setLoggingFileName(loggingFileName);
		
		String currentTest = TestFactory.DOWNSTREAMTHROUGHPUT;
		while(true)
		{
			for (TestDescription td : mTestDescription) {
				if(td.type == null) continue;
				if(!td.type.equals(currentTest)) continue;
				
				manualTestExecutor.addRequestedTest(td);
				// check if a stop command has been received
				if (!run.get()) {
					manualTestExecutor.cancelNotification();
					SKLogger.d(this, "Manual test interrupted by the user.");
					break;
				}
				com.samknows.measurement.test.TestResult tr = new com.samknows.measurement.test.TestResult();
				ObservableExecutor oe = new ObservableExecutor(manualTestExecutor, td, tr);
				Thread t = new Thread(oe);
				t.start();
				while (true) {
					long runningTime = System.currentTimeMillis();
					try {
						t.join(100);
						if (!t.isAlive())
							break;
					} catch (InterruptedException ie) {
						SKLogger.e(this, ie.getMessage());
					}
					for (JSONObject pm : progressMessage(td, manualTestExecutor)) {
						msg = mHandler.obtainMessage(0, pm);
						mHandler.sendMessage(msg);
					}
					runningTime = System.currentTimeMillis() - runningTime;
					if(runningTime < 500){
						try {
							Thread.sleep(500-runningTime);
						} catch (InterruptedException e) {

						}						
					}

					
					if (!run.get()) {
						// stop By user
						break;
					}
				}
				// one test finish

				List<JSONObject> currResults = new ArrayList<JSONObject>();
				for (String out : tr.results) {
					currResults.addAll(TestResult.testOutput(out));
				}
				for (JSONObject cr : currResults) {
					msg = mHandler.obtainMessage(0, cr);
					mHandler.sendMessage(msg);
				}
				testsResults.addAll(currResults);
			}

			if(currentTest.equals(TestFactory.DOWNSTREAMTHROUGHPUT)){
				currentTest = TestFactory.UPSTREAMTHROUGHPUT;
			}
			else if(currentTest.equals(TestFactory.UPSTREAMTHROUGHPUT)){
				currentTest = TestFactory.LATENCY;
			}
			else if(currentTest.equals(TestFactory.LATENCY)){
				break;
			}
		}
		
		manualTestExecutor.stop();
		long endTime = System.currentTimeMillis();
		
		manualTestExecutor.save("manual_test");

		// Gather data from collectors
		for (BaseDataCollector collector : tc.config.dataCollectors) {
			if (collector.isEnabled) {
				for (JSONObject o : collector.getPassiveMetric()) {
					// save metric
					passiveMetrics.add(o);
				}
			}
		}
		
		// insert batch in the database
		try {
			batch.put(TestBatch.JSON_DTIME, startTime);
			batch.put(TestBatch.JSON_RUNMANUALLY, "1");
		} catch (JSONException je) {
			SKLogger.e(this,
					"Error in creating test batch object: " + je.getMessage());
		}

		// insert the results in the database only if we didn't receive a stop
		// command
		if (run.get()) {
			db.insertTestBatch(batch, testsResults, passiveMetrics);
		}
		else { //stop by user
			sSetIsExecuting(false);
			return;			
		}

		// Send completed message to the interface
		msg = mHandler.obtainMessage(0);
		JSONObject jtc = new JSONObject();
		try {
			Thread.sleep(1000);
			jtc.put(TestResult.JSON_TYPE_ID, "completed");
			msg.obj = jtc;

		} catch (JSONException je) {
			SKLogger.e(this, "Error in creating json object: " + je.getMessage());
		} catch (InterruptedException e) {
			SKLogger.e(
					this,
					"Sleep interrupted in the manual test view: "
							+ e.getMessage());
		}
		mHandler.sendMessage(msg);
		
		// Make history data & save history.csv
		String[] history_value = new String[ExportFile.HISTORY_FIELDS.length];
		{
			CustomTimestamp stamp = new CustomTimestamp(startTime);
			String startString = stamp.toString();
			history_value[0] = startString;				// Start Time
			
			stamp.setTime(endTime);
			String endString = stamp.toString();
			history_value[1] = endString;				// End Time
			
			CollectDeviceInfo device = CollectDeviceInfo.getInstance();
			history_value[2] = device.net;				// NET
			history_value[3] = ""+device.getCellID();		// Cell ID 
			history_value[4] = "0";						// PCI
			history_value[5] = "0";			// SINR
			history_value[6] = "0";			// RSRP
			history_value[7] = "0";			// RSRQ
			history_value[8] = ""+device.getRSSI();			// RSSI
			
			
			for(JSONObject json : testsResults){
				try {
					String id_string = json.getString(TestResult.JSON_TYPE_ID);
					int id = Integer.parseInt(id_string);
					if(id == TestResult.LATENCY_TEST_ID){
						history_value[9] = ""+json.getDouble(TestResult.JSON_RESULT);	// Latency
					}
					else if(id == TestResult.PACKETLOSS_TEST_ID){
						history_value[10] = ""+json.getDouble(TestResult.JSON_RESULT);	// Packet loss
					}
					else if(id == TestResult.DOWNLOAD_TEST_ID){
						history_value[11] = ""+json.getDouble(TestResult.JSON_PEAK);	// Max_DL
						history_value[13] = ""+json.getDouble(TestResult.JSON_RESULT);	// Avg_DL
					}
					else if(id == TestResult.UPLOAD_TEST_ID){
						history_value[12] = ""+json.getDouble(TestResult.JSON_PEAK);	// Max_UL
						history_value[14] = ""+json.getDouble(TestResult.JSON_RESULT);	// Avg_UL
					}
				} catch (JSONException e) {
					Log.e(TAG,""+e.getMessage());
				}
			}
			
//			history_value[15] = ""+device.latitude;			// GPS Latitude
//			history_value[16] = ""+device.longitude;		// GPS Longitude
//			history_value[17] = ""+device.accuracy;			// GPS Accuracy
			
			history_value[15] = ""+manualTestExecutor.getResultItem("Latitude");			// GPS Latitude
			history_value[16] = ""+manualTestExecutor.getResultItem("Longitude");		// GPS Longitude
			history_value[17] = ""+manualTestExecutor.getResultItem("Accuracy");			// GPS Accuracy

		}
		ExportFile.appendToCSV("history.csv", ExportFile.HISTORY_FIELDS, history_value);
		

		try {
			SubmitTestResultsAction action = new SubmitTestResultsAction(ctx);
			action.setTestInfo(new CustomTimestamp(startTime){}.toString(), new CustomTimestamp(endTime){}.toString(), ""+time+"_"+imsi+"_"+model+".tsv",imsi,isRepeatTest);
			action.execute();
		} catch (Throwable t) {
			SKLogger.e(this, "Submit result. ", t);
		}
		
	
//		if(!SK2AppSettings.getInstance().isServiceEnabled()){
//			MainService.force_poke(ctx);
//		}
		
		sSetIsExecuting(false);
	}
	
	private static void sSetIsExecuting(boolean bValue) {
		isExecuting = bValue;
	}
	
	public String getTargetAddress(){
		if(mTestDescription == null) return null;
		
		for(TestDescription td : mTestDescription){
			for(Param param : td.params){
				String name = param.getName();
				String value = param.getValue();
				if(name.equalsIgnoreCase("target")){
					return value;
				}
			}
		}
		return null;
	}
	
	public int getSpeedTestPost(){
		return 8000;
	}
	

	private class ObservableExecutor implements Runnable {
		public TestExecutor te;
		TestDescription td;
		com.samknows.measurement.test.TestResult tr;

		public ObservableExecutor(TestExecutor te, TestDescription td,
				com.samknows.measurement.test.TestResult tr) {
			this.te = te;
			this.td = td;
			this.tr = tr;
		}

		@Override
		public void run() {
			te.executeTest(td, tr);
		}

	}

	static private List<JSONObject> progressMessage(TestDescription td,
			TestExecutor te) {
		List<JSONObject> ret = new ArrayList<JSONObject>();
		List<String> tests = new ArrayList<String>();

		if (td.type.equals(TestFactory.DOWNSTREAMTHROUGHPUT)) {
			tests.add("" + TestResult.DOWNLOAD_TEST_ID);
		} else if (td.type.equals(TestFactory.UPSTREAMTHROUGHPUT)) {
			tests.add("" + TestResult.UPLOAD_TEST_ID);
		} else if (td.type.equals(TestFactory.LATENCY)) {
			tests.add("" + TestResult.LATENCY_TEST_ID);
			tests.add("" + TestResult.PACKETLOSS_TEST_ID);
			//tests.add("" + TestResult.JITTER_TEST_ID);
		}
		try {
			for (String t : tests) {
				JSONObject c = new JSONObject();
				c.put(TestResult.JSON_TYPE_ID, "test");
				c.put(TestResult.JSON_TESTNUMBER, t);
				c.put(TestResult.JSON_STATUS_COMPLETE, te.getProgress());
				c.put(TestResult.JSON_SPEED, te.getSpeed());
				c.put(TestResult.JSON_INTERVAL, te.getInterval());
				c.put(TestResult.JSON_PEAK, te.getPeak());
				c.put(TestResult.JSON_LOSS, te.getLoss());
				ret.add(c);
			}
		} catch (JSONException je) {
			SKLogger.e(
					ManualTest.class,
					"Error in creating JSON progress object: "
							+ je.getMessage());
		}
		return ret;
	}

	// It stops the test from being executed
	public void stop() {
		
		run.set(false);
	}
	
	public void setIsRepeatTest(boolean isRepeat){
		isRepeatTest = isRepeat;
	}


	public void setHandler(Handler handler) {
		mHandler = handler;
	}
}
