package com.samknows.measurement;

import java.io.File;
import java.util.Timer;
import java.util.TimerTask;

import com.samknows.libcore.R;
import com.samknows.libcore.SKConstants;
import com.samknows.libcore.SKLogger;
import com.samknows.measurement.environment.CellTowersDataCollector;
import com.samknows.measurement.schedule.ServerInfo;
import com.samknows.measurement.storage.ExportFile;
import com.samknows.measurement.test.TestResultsManager;
import com.samknows.measurement.util.OtherUtils;

import android.app.Application;
import android.content.SharedPreferences;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.util.Log;

public class SKApplication extends Application{
	private static final String TAG = SKApplication.class.getSimpleName();
	
	public static final int MSG_BGTEST_START = 1000;
	

	private static SKApplication sAppInstance = null;
	
	public SKApplication() {
		super();

		sAppInstance = this;

		SKConstants.RStringQuit = R.string.quit;
		SKConstants.RStringReallyQuit = R.string.really_quit;
		SKConstants.RStringYes = R.string.yes;
		SKConstants.RStringNoDialog = R.string.no_dialog;

		SKConstants.PREF_KEY_USED_BYTES = "used_bytes";
		SKConstants.PREF_DATA_CAP = "data_cap_pref";
		SKConstants.PROP_TEST_START_WINDOW_RTC = "test_start_window_in_millis_rtc";
	}

	@Override
	public void onCreate() {
		super.onCreate();
		
		File storage = getExternalCacheDir();
		if (storage == null) {
			storage = getCacheDir();
		}

		SKLogger.setStorageFolder(storage);
		TestResultsManager.setStorage(storage);
		ExportFile.setStorage(storage);
		SK2AppSettings.create(this);
		CachingStorage.create(this);
		CollectDeviceInfo.create(this);
		
		// Start monitoring for cell tower signal strength etc....
		// We need to do this, as Android does not allow us to query this information synchronously.
		CellTowersDataCollector.sStartToCaptureCellTowersData(this);
		
	}


	public static SKApplication getAppInstance() {
		// application instance only one.
		return sAppInstance;
	}
	
	// Todo Item...
	private ServerInfo bestServer;
	public void setBestServer(ServerInfo server){
		bestServer = server;
	}
	

	// Network type results querying...
	public enum eNetworkTypeResults {
		eNetworkTypeResults_Any,
		eNetworkTypeResults_Mobile,
		eNetworkTypeResults_WiFi
	};

	private static eNetworkTypeResults sNetworkTypeResults = eNetworkTypeResults.eNetworkTypeResults_Mobile;

	public static eNetworkTypeResults getNetworkTypeResults() {
		return sNetworkTypeResults;
	}

	public static void setNetworkTypeResults(eNetworkTypeResults networkTypeResults) {
		sNetworkTypeResults = networkTypeResults;
	}

	// Override this, if you want your application to support a 1-day result view.
	public boolean supportOneDayResultView() {
		return false;
	}

	// Get the class of the main activity!
	public Class getTheMainActivityClass() {
		return null;
	}

	// Return the About screen title.
	public String getAboutScreenTitle() {
		return getApplicationContext().getString(R.string.about);
	}

	public boolean hideJitter() {
		return false;
	}

	public boolean hideJitterLatencyAndPacketLoss() {
		return false;
	}

	public boolean allowUserToSelectTestToRun() {
		// Run all tests!
		return false;
	}

	public boolean isExportMenuItemSupported() {
		return false;
	}
	
	// Datacap enabling/disabling
	
	public boolean canDisableDataCap () {
		return false;
	}
	
	// Datacap - enable/disable (managed via the SKAPreferenceActivity)
	final public boolean getIsDataCapEnabled() {
    	if (canDisableDataCap () == false) {
    		// Can't disable the datacap in this version of the app - so in this
    		// case, the data cap is always enabled.
    		return true;
    	}
    	
		// The value is saved/restored automatically through PreferenceManager.
		SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
		return p.getBoolean(SKConstants.PREF_DATA_CAP_ENABLED, true);
	}
	
	public boolean isSocialMediaExportSupported() {
		return false;
	}
	
	// Support throttle query supported?
	public boolean isThrottleQuerySupported() {
		return false;
	}
	
	private ManualTest mManualTest;
	public void setManualTest(ManualTest mt){
		mManualTest = mt;
	}
	public ManualTest getManualTest(){
		return mManualTest;
	}
	
	private Handler mActivityHandler = null;
	// must be set Activity resume, pause
	public void setActivityHandler(Handler handler){
		mActivityHandler = handler;
	}
	public Handler getActivityHandler(){
		return mActivityHandler;
	}
	
}
