package com.samknows.measurement.schedule;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.samknows.libcore.SKConstants;
import com.samknows.measurement.CachingStorage;
import com.samknows.measurement.TestParamsManager;
import com.samknows.measurement.util.IdGenerator;
import com.samknows.measurement.util.OtherUtils;
import com.samknows.measurement.util.XmlUtils;
import com.samknows.tests.Param;
import com.samknows.tests.TestFactory;


public class TestDescription implements Serializable{
	private static final long serialVersionUID = 1L;
	
	//xml tag definition
	public static final String XML_TYPE = "type";
	public static final String XML_TEST_ID = "test-id";
	public static final String XML_CONDITION_GROUP_ID = "condition-group-id";
	public static final String XML_DISPLAY_NAME = "displayName";
	public static final String XML_IS_PRIMARY = "isPrimary";
	public static final String XML_TIME = "time";
	public static final String XML_PARAMS = "params";
	public static final String XML_PARAM = "param";
	public static final String XML_PARAM_NAME = "name";
	public static final String XML_PARAM_VALUE = "value";
	public static final String XML_POSITION = "position";
	public static final String XML_FIELD = "field";
	public static final String XML_FAILOVER_PARAMS = "failover-params";
	

	public static final long NO_START_TIME = -1;
	
	public long id = IdGenerator.generate();

	public String type;// TestFactory
	public String displayName;
	public List<Param> params;
	
	
	
	public static TestDescription parseJSON(String name, JSONObject object) 
	{
		TestDescription td = new TestDescription();
		td.displayName = name;
		if(td.displayName.equalsIgnoreCase("download")){
			td.type = TestFactory.DOWNSTREAMTHROUGHPUT;
		}
		else if(td.displayName.equalsIgnoreCase("upload")){
			td.type = TestFactory.UPSTREAMTHROUGHPUT;
		}
		else if(td.displayName.equalsIgnoreCase("latency")){
			td.type = TestFactory.LATENCY;
		}
		
		td.params = new ArrayList<Param>();

		try {
			JSONArray names = object.names();
			for(int i=0; i<names.length(); i++){
				String key_name = names.getString(i);
				String value = object.getString(key_name);
				td.params.add(new Param(key_name, value));
			}	
		}
		catch(JSONException e){
			e.printStackTrace();
		}

		return td;
	}
	
	
	
	
	public boolean canExecute() {
		TestParamsManager manager = CachingStorage.getInstance().loadParamsManager(); 
		boolean isClosestParamSet = manager == null ? false : manager.hasParam("closest");
		return isClosestParamSet || type.equalsIgnoreCase(SKConstants.TEST_TYPE_CLOSEST_TARGET);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (id ^ (id >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		TestDescription other = (TestDescription) obj;
		if (id != other.id)
			return false;
		return true;
	}
	
	public String getTypeString(){
		return TestFactory.getTestString(type, params);
	}
}
