package com.samknows.measurement.statemachine;

import java.util.Map;


import com.samknows.measurement.MainService;
import com.samknows.measurement.SK2AppSettings;
import com.samknows.measurement.statemachine.state.BaseState;
import com.samknows.measurement.statemachine.state.DownloadConfigAnonymousState;
import com.samknows.measurement.statemachine.state.ExecuteScheduledTestQueueState;
import com.samknows.measurement.statemachine.state.NoneState;
import com.samknows.measurement.statemachine.state.RunInitTestsState;

public abstract class Transition {
	
	public static Transition create(SK2AppSettings as){
//		if(as.onlyDownload){
//			return new TransitionOnlyDownload();
//		}
		if(as.background_test){
			return new TransitionBackground();
		}
		return new TransitionAnonymous();
	}
	
	public abstract String getType();
	protected abstract Map<State, String[]> getTransition();
	
	public State getNextState(State state, StateResponseCode code) {
		String[] state_transiction = getTransition().get(state);
		if(state_transiction == null){
			throw new RuntimeException(getType() +" state machine doesn't define state: "+ state);
		}
		for (String s : state_transiction) {
			if (s.startsWith(code.toString())) {
				return State.valueOf(s.split(":")[1]);
			}
		}
		throw new RuntimeException(getType() +" does not define " +state +" ->" + code);
		
	}
	
	public static BaseState createState(State state, MainService ctx) {
		switch (state) {
		case NONE: return new NoneState(ctx);
		case INITIALISE : return null;
		case INITIALISE_ANONYMOUS: return null;
		case ACTIVATE : return null;
		case ASSOCIATE : return null;
		case CHECK_CONFIG_VERSION : return null;
		case DOWNLOAD_CONFIG : return null;
		case DOWNLOAD_CONFIG_ANONYMOUS: return new DownloadConfigAnonymousState(ctx);
		case RUN_INIT_TESTS : return new RunInitTestsState(ctx);
		case EXECUTE_QUEUE : return new ExecuteScheduledTestQueueState(ctx);
		case SUBMIT_RESULTS : return null;
		case SUBMIT_RESULTS_ANONYMOUS : return null;
		case SHUTDOWN:
		}
		throw new RuntimeException("unimplemented state: " + state);
	}
}
