package com.samknows.measurement.storage;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.util.Log;

import com.samknows.libcore.SKLogger;
import com.samknows.measurement.CollectDeviceInfo;
import com.samknows.measurement.SK2AppSettings;
import com.samknows.measurement.environment.CellTowersData;
import com.samknows.measurement.net.SubmitTestResultsAction;
import com.samknows.measurement.schedule.TestDescription;
import com.samknows.measurement.schedule.condition.Condition;
import com.samknows.tests.HttpTest;
import com.samknows.tests.Param;
import com.samknows.tests.TestFactory;

public class ResultsContainer {
	
	private static final String TAG = ResultsContainer.class.getSimpleName();

	public static final String JSON_TESTS = "tests";
	public static final String JSON_CONDITIONS = "conditions";
	public static final String JSON_METRICS = "metrics";
	public static final String JSON_REQUESTED_TESTS = "requested_tests";
	public static final String JSON_CONDITION_BREACHES = "condition_breaches";
	private List<JSONObject> mTests = new ArrayList<JSONObject>();
	private List<JSONObject> mConditions = new ArrayList<JSONObject>();
	private List<JSONObject> mMetrics = new ArrayList<JSONObject>();
	private Map<String, Object> mExtra = new LinkedHashMap<String, Object>();
	private JSONArray mRequestedTests = new JSONArray();
	private Map<String, String> mConditionBreaches = new HashMap<String,String>();
	
	public ResultsContainer(){
		
	}
	
	public ResultsContainer(Map<String, String> extra){
		mExtra.putAll(extra);
	}
	
	public void addTest(JSONObject test){
		mTests.add(test);
	}
	
	public void addTest(List<JSONObject> tests){
		mTests.addAll(tests);
	}
	
	public void addCondition(JSONObject condition){
		mConditions.add(condition);
	}
	
	public void addCondition(List<JSONObject> conditions){
		mConditions.addAll(conditions);
	}
	
	public void addMetric(JSONObject metric){
		mMetrics.add(metric);
	}
	
	public void addMetric(List<JSONObject> metrics){
		mMetrics.addAll(metrics);
	}

	private int getNumberOfThreadsForHttpTestDescription(
			TestDescription testDescription) {
		int numberOfThreads = 1;
		for (Param curr : testDescription.params) {
			String paramName = curr.getName();
			// The parameter value is "numberofthreads", the internal value is "numberOfThreads"
			if (paramName.equalsIgnoreCase(TestFactory.NTHREADS)) {
				String paramValue = curr.getValue();
				numberOfThreads = Integer.parseInt(paramValue);
			}
		}
		return numberOfThreads;
	}
	
	public void addFailedCondition(String condition) {
		mConditionBreaches.put(condition, condition);
	}
	
	public void addFailedCondition(Condition condition) {
		
		// "condition_breach"

		// get e.g. "NETACTIVITY"
		String theString = condition.getConditionStringForReportingFailedCondition();
		// Using a map, prevents duplicates!
		mConditionBreaches.put(theString, theString);
	}
	
	public void addRequestedTest(TestDescription td){
	
		mRequestedTests.put(td.getTypeString());
	}

	public void addExtra(String key, String value){
		mExtra.put(key, value);
	}
	
	
	public JSONObject allItem;
	private int getJsonInt(JSONObject o, String name){
		try {
			return o.getInt(name);
		} catch (JSONException e) {
			
		}
		return 0;
	}
	private long getJsonLong(JSONObject o, String name){
		try {
			return o.getLong(name);
		} catch (JSONException e) {
			
		}
		return 0;
	}
	private double getJsonDouble(JSONObject o, String name){
		try {
			return o.getDouble(name);
		} catch (JSONException e) {
			
		}
		return 0.0f;
	}

	
	
	public JSONObject getJSON(){
		mExtra.putAll(SK2AppSettings.getSK2AppSettingsInstance().getJSONExtra());
		JSONObject ret = new JSONObject(mExtra);
		JSONArray tests = new JSONArray();
		JSONArray conditions = new JSONArray();
		JSONArray metrics = new JSONArray();
		
		for(JSONObject t: mTests){
			tests.put(t);
		}
		for(JSONObject c: mConditions){
			conditions.put(c);
		}
		for(JSONObject m: mMetrics){
			metrics.put(m);
		}
		
		try{
			if(mRequestedTests.length() > 0){
				ret.put(JSON_REQUESTED_TESTS, mRequestedTests);
			}
			if (mConditionBreaches.size() > 0) {
            	JSONArray conditionBreaches = new JSONArray();
            
                Iterator<Entry<String, String>> it = mConditionBreaches.entrySet().iterator();
                while (it.hasNext()) {
                    Entry<String,String> pairs = it.next();
                    conditionBreaches.put(pairs.getKey());
                }
            	
				ret.put(JSON_CONDITION_BREACHES, conditionBreaches);
			}
			ret.put(JSON_TESTS, tests);
			ret.put(JSON_METRICS, metrics);
			ret.put(JSON_CONDITIONS, conditions);
		}catch(JSONException je){
			SKLogger.e(this, "Error in creating a JSONObject: " + je.getMessage() );
			ret = null;
		}
		
		try {
			allItem = new JSONObject();
			CollectDeviceInfo device = CollectDeviceInfo.getInstance();
			
			for(JSONObject t: mTests){
				String type = t.getString("type");
				Log.e(TAG,"type:"+type);
				if(type.equals(HttpTest.DOWNSTREAMMULTI) || type.equals(HttpTest.DOWNSTREAMSINGLE)){

					long avg_throughput = (long)t.getInt("bytes_sec_after_1sec")*8;
					long max_throughput = (long)t.getInt("peak_bytes_sec")*8;
					double avg_throughput_mbit = avg_throughput/1000000d;
					double max_throughput_mbit = max_throughput/1000000d;
					
					allItem.put("Avg_DL_Throughput", avg_throughput_mbit);
					allItem.put("Max_DL_Throughput", max_throughput_mbit);
					allItem.put("GET_Result", t.getBoolean("success")?"success":"fail");
					allItem.put("GET_Thread", t.getInt("number_of_threads"));
					allItem.put("GET_Warmup_Bytes", t.getInt("warmup_bytes"));
					allItem.put("GET_Warmup_Time", t.getLong("warmup_time"));
					allItem.put("GET_Byte_Sec", t.getInt("bytes_sec"));
					allItem.put("GET_Transfer_Time", t.getLong("transfer_time"));
					allItem.put("GET_Byte_Total", t.getInt("transfer_bytes"));

				}
				else if(type.equals(HttpTest.UPSTREAMMULTI) || type.equals(HttpTest.UPSTREAMSINGLE)){
					
					long avg_throughput = (long)t.getInt("bytes_sec_after_1sec")*8;
					long max_throughput = (long)t.getInt("peak_bytes_sec")*8;
					double avg_throughput_mbit = avg_throughput/1000000d;
					double max_throughput_mbit = max_throughput/1000000d;
					
					allItem.put("Avg_UL_Throughput", avg_throughput_mbit);
					allItem.put("Max_UL_Throughput", max_throughput_mbit);
					allItem.put("POST_Result", t.getBoolean("success")?"success":"fail");
					allItem.put("POST_Thread", t.getInt("number_of_threads"));
					allItem.put("POST_Warmup_Bytes", t.getInt("warmup_bytes"));
					allItem.put("POST_Warmup_Time", t.getLong("warmup_time"));
					allItem.put("POST_Byte_Sec", t.getInt("bytes_sec"));
					allItem.put("POST_Transfer_Time", t.getLong("transfer_time"));
					allItem.put("POST_Byte_Total", t.getInt("transfer_bytes"));
					
				}
				else if(type.equals("JUDPLATENCY")){
					
					int received = t.getInt("received_packets");
					int loss = t.getInt("lost_packets");
					double ratio = loss/(double)(received+loss);
					ratio *= 100d;
					
					int latency_us = t.getInt("rtt_avg");
					double latency_ms = latency_us/1000d;
					
					allItem.put("Latency", latency_ms);
					allItem.put("Packet_Loss_Ratio", ratio);
					allItem.put("RTT_Result", t.getBoolean("success")?"success":"fail");
					allItem.put("RTT_AVG", t.getInt("rtt_avg"));
					allItem.put("RTT_MIN", t.getInt("rtt_min"));
					allItem.put("RTT_MAX", t.getInt("rtt_max"));
					allItem.put("RTT_STDDEV", t.getInt("rtt_stddev"));
					allItem.put("Received_Packets", t.getInt("received_packets"));
					allItem.put("Lost_Packet", t.getInt("lost_packets"));
				}
			}
			
			for(JSONObject m: mMetrics){
				String type = m.getString("type");
				//Log.e(TAG,"type:"+type);
				if(type.equals("location")){
					
					allItem.put("Location_Type", m.getString("location_type"));
					allItem.put("Longitude", m.getDouble("longitude"));
					allItem.put("Latitude", m.getDouble("latitude"));
					allItem.put("Accuracy", m.getDouble("accuracy"));
				}
				else if(type.equals("network_data")){
					
					allItem.put("Phone_Type", m.getString("phone_type"));
					allItem.put("Network_Type", m.getString("network_type"));
					allItem.put("Connected", m.getBoolean("connected")?"TRUE":"FALSE");
					allItem.put("Active_Network_Type", m.getString("active_network_type"));
					allItem.put("Network_Operator_Name", m.getString("network_operator_name"));
					allItem.put("Network_Operator_Code", getJsonInt(m,"network_operator_code"));
					allItem.put("SIM_Operator_Name", m.getString("sim_operator_name"));
					allItem.put("SIM_Operator_Code", getJsonInt(m,"sim_operator_code"));
					allItem.put("Roaming", m.getBoolean("roaming")?"TRUE":"FALSE");
					
				}
				else if(type.equals("phone_identity")){
					
					allItem.put("Model", m.getString("model"));
					allItem.put(SubmitTestResultsAction.MANUFACTURER, m.getString("manufacturer"));
					allItem.put("OS_Type", m.getString("os_type"));
					allItem.put("OS_Version", m.getString("os_version"));
					
				}
				else if(type.equals("net_usage")){
					
					allItem.put("Total_TX_Bytes", m.getInt("total_tx_bytes"));
					allItem.put("Mobile_TX_Bytes", m.getInt("mobile_tx_bytes"));
					allItem.put("APP_TX_Bytes",	m.getInt("app_tx_bytes"));
					allItem.put("Total_RX_Bytes", m.getInt("total_rx_bytes"));
					allItem.put("Mobile_RX_Bytes", m.getInt("mobile_rx_bytes"));
					allItem.put("APP_RX_Bytes", m.getInt("app_rx_bytes"));
					allItem.put("Duration", m.getLong("duration"));
					
				}
				else if(type.equals("gsm_cell_location") && !device.net.equals("WIFI")){

//					if(m.getInt("lte_pci") != 0){
//						allItem.put("LTE_PCI", m.getInt("lte_pci"));
//					}
					allItem.put("LTE_CellID", m.getInt(CellTowersData.JSON_CELL_TOWER_ID));
					allItem.put("GSM_CellID", m.getInt(CellTowersData.JSON_CELL_TOWER_ID));
					allItem.put("GSM_LAC", m.getInt(CellTowersData.JSON_LOCATION_AREA_CODE));
					allItem.put("GSM_PSC", m.getInt(CellTowersData.JSON_UMTS_PSC));

				}
				else if(type.equals("cdma_cell_location") && !device.net.equals("WIFI")){
					
					allItem.put("CDMA_BSID", m.getInt(CellTowersData.JSON_BASE_STATION_ID));
					allItem.put("CDMA_SID", m.getInt(CellTowersData.JSON_SYSTEM_ID));
					allItem.put("CDMA_NID", m.getInt(CellTowersData.JSON_NETWORK_ID));
					
				}
				else if(type.equals("cell_strength_data") && !device.net.equals("WIFI")){
					
					allItem.put("LTE_RSSI", m.getInt("lte_rssi"));
					allItem.put("LTE_RSRQ", m.getInt("lte_rsrq"));
					allItem.put("LTE_RSRP", m.getInt("lte_rsrp"));
					allItem.put("LTE_SINR", m.getDouble("lte_sinr"));
					allItem.put("GSM_RSSI", m.getInt("gsm_rssi"));
					allItem.put("CDMA_RSSI", m.getInt("cdma_rssi"));
					allItem.put("CDMA_ECIO", m.getInt("cdma_ecio"));
					allItem.put("EVDO_RSSI", m.getInt("evdo_rssi"));
					allItem.put("EVDO_ECIO", m.getInt("evdo_ecio"));
				}
			}
			
		} catch (JSONException e) {
			Log.e(TAG,""+e.getMessage());
		}
		
		return ret;
		//return allItem;
	}
	
	public String getResultItem(String itemName){
		if(allItem == null) return "";
		try {
			String result = null;
			if(itemName.equals("Longitude") || itemName.equals("Latitude") || itemName.equals("Accuracy")){
				result = ""+allItem.getDouble(itemName);
			}
			else {
				result = allItem.getString(itemName);	
			}
			return result;
		} catch (JSONException e) {
			
		}
		return "";
	}
	
}

