package com.samknows.measurement.util;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.io.ByteArrayInputStream;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.telephony.TelephonyManager;
import android.util.Log;

import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.pm.Signature;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import javax.security.auth.x500.X500Principal;


import com.samknows.libcore.SKLogger;
import com.samknows.measurement.SK2AppSettings;
import com.samknows.measurement.DeviceDescription;
import com.samknows.measurement.MainService;
import com.samknows.libcore.R;
import com.samknows.measurement.environment.PhoneIdentityDataCollector;
import com.samknows.measurement.statemachine.State;

public class OtherUtils {
	
	private static final String TAG = OtherUtils.class.getSimpleName();
	
	
	public static String formatToBytes(long bytes) {
		double data = bytes;
		if (data > 1024*1024) { 
			data /= 1024d;
			data /= 1024d;
			return String.format("%.2fMB", data);
		} else if (data > 1024) {
			data /= 1024d;
			return String.format("%.2fKB", data);
		} else {
			return bytes + "B";
		}
	}

	public static String formatToBits(long bytes) {
		double data = bytes;
		data *= 8;
		if (data > 1000*1000) { 
			data /= 1000d;
			data /= 1000d;
			return String.format("%.2fMb", data);
		} else if (data > 1000) {
			data /= 1000d;
			return String.format("%.2fKb", data);
		} else {
			return bytes + "b";
		}
	}

	public static void reschedule(Context ctx, long timeDurationMilliseconds){
		long actualSystemTimeMilliseconds;
		if(SK2AppSettings.getInstance().isWakeUpEnabled()){
			actualSystemTimeMilliseconds = rescheduleWakeup(ctx, timeDurationMilliseconds);
		}else{
			actualSystemTimeMilliseconds = rescheduleRTC(ctx, timeDurationMilliseconds);
		}
		SKLogger.d(ctx,  "Rescheduled to " + TimeUtils.logString(actualSystemTimeMilliseconds) + ", "+ (actualSystemTimeMilliseconds - System.currentTimeMillis()) +" ms from now...");
	}

	public static long rescheduleRTC(Context ctx, long time) {

		SKLogger.d(OtherUtils.class, "+++++DEBUG+++++ rescheduleRTC time=" + time);

		// time = 60000; Logger.d(OtherUtils.class, "+++++DEBUG+++++ rescheduleRTC REMOVE ME!! forced time to time=" + time);

		//time = checkRescheduleTime(time);
		SKLogger.d(OtherUtils.class, "+++++DEBUG+++++ schedule RTC for " + time/1000 + "s from now");
		PendingIntent intent = PendingIntent.getService(ctx, 0, new Intent(ctx, MainService.class), PendingIntent.FLAG_UPDATE_CURRENT);
		
		AlarmManager manager = (AlarmManager) ctx.getSystemService(Context.ALARM_SERVICE);
		long systemTimeMilliseconds = System.currentTimeMillis() + time;

		// AlarmManager.RTC - This does NOT wake-up the device if it is asleep.
		manager.set(AlarmManager.RTC, systemTimeMilliseconds, intent);
		Log.e(ctx.getClass().getSimpleName(),"rescheduleRTC ########################################## time:"+time);
		
		SK2AppSettings.getInstance().saveNextRunTime(systemTimeMilliseconds);
		return systemTimeMilliseconds;
	}

	public static void cancelAlarm(Context ctx){
		SKLogger.d(OtherUtils.class, "+++++DEBUG+++++ cancelAlarm");

		AlarmManager manager = (AlarmManager) ctx.getSystemService(Context.ALARM_SERVICE);
		if(PendingIntent.getService(ctx,0, new Intent(ctx, MainService.class), PendingIntent.FLAG_NO_CREATE) == null){
			SKLogger.d(ctx, "There is no pending intent for the service");
		}
		PendingIntent intent = PendingIntent.getService(ctx, 0, new Intent(ctx, MainService.class), PendingIntent.FLAG_UPDATE_CURRENT);
		manager.cancel(intent);
		Log.e(ctx.getClass().getSimpleName(),"cancelAlarm ##########################################");
		
		//SK2AppSettings.getInstance().saveNextRunTime(SKConstants.NO_NEXT_RUN_TIME);
	}

	public static long rescheduleWakeup(Context ctx, long time) {

		Calendar c = Calendar.getInstance();

		SKLogger.d(OtherUtils.class, "+++++DEBUG+++++ schedule RTC_WAKEUP for " + time/1000 + "s from now");
		PendingIntent intent = PendingIntent.getService(ctx, 0, new Intent(ctx, MainService.class), PendingIntent.FLAG_UPDATE_CURRENT);
		AlarmManager manager = (AlarmManager) ctx.getSystemService(Context.ALARM_SERVICE);
		long systemTimeMilliseconds = System.currentTimeMillis() + time;
		c.setTimeInMillis(systemTimeMilliseconds);
		
		// For Debug
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy.MM.dd HH:mm");
		String timeString = formatter.format(c.getTime());
		Log.e(ctx.getClass().getSimpleName(),"RTC_WAKEUP time "+timeString);
		
		// AlarmManager.RTC_WAKEUP - This DOES wake-up the device if it is asleep.
		manager.set(AlarmManager.RTC_WAKEUP, systemTimeMilliseconds, intent);
		Log.e(ctx.getClass().getSimpleName(),"rescheduleWakeup time:"+time+" ##########################################");
		
		SK2AppSettings.getInstance().saveNextRunTime(systemTimeMilliseconds);
		return systemTimeMilliseconds;
	}
	
	public static long getNextRunTime(){
		
		// 1. get closest test time ( 07~09, 09~17, 17~20)
		Calendar rightnow = Calendar.getInstance();
		int cur_hour = rightnow.get(Calendar.HOUR_OF_DAY);
		int next_hour = 0;
		if(cur_hour < 7){
			next_hour = 7;
		}
		else if(cur_hour < 9){
			next_hour = 9;
		}
		else if(cur_hour < 17){
			next_hour = 17;
		}
		else {
			next_hour = 7;
			rightnow.add(Calendar.DATE, 1);
		}
		rightnow.set(Calendar.HOUR_OF_DAY, next_hour);
		
		// 2. random range
		int during = 0;
		if(next_hour == 7) during = 2;
		else if(next_hour == 9) during = 8;
		else if(next_hour == 17) during = 3;
		
		
		// 3. get random count
		Random r = new Random();
		int random_count = r.nextInt(during*60*60*1000);
		rightnow.add(Calendar.MILLISECOND, random_count);
		
		
		// 4. next random
		long next_time = rightnow.getTimeInMillis() - System.currentTimeMillis();
		Log.d(TAG,"after next time:"+next_time);
		Log.d(TAG,"runtime:"+rightnow.toString());
		
		return next_time;
	}

	//if the reschedule time is less than the testStart window play it safe an reschedule the main service for the 
	//the rescheduleTime
	private static long checkRescheduleTime(long time){
		SK2AppSettings a = SK2AppSettings.getSK2AppSettingsInstance();
		
		long ret = time;
	
		if(time <= a.getTestStartWindow()){
			SKLogger.w(OtherUtils.class, "reschedule time less than testStartWindow ("+a.getTestStartWindow()+"), changing it to: "+ a.rescheduleTime/1000+"s.");
			ret = a.rescheduleTime;
		}
		return ret;
	}

	public String getLocalIpAddress() {
		try {
			for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
				NetworkInterface intf = en.nextElement();
				for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements();) {
					InetAddress inetAddress = enumIpAddr.nextElement();
					if (!inetAddress.isLoopbackAddress()) {
						return inetAddress.getHostAddress().toString();
					}
				}
			}
		} catch (SocketException ex) {
			SKLogger.e(OtherUtils.class, "failed to get ip address", ex);
		}
		return null;
	}

	public static boolean isRoaming(Context ctx) {
		TelephonyManager tm = (TelephonyManager) ctx.getSystemService(Context.TELEPHONY_SERVICE);
		return tm.isNetworkRoaming();
	}

	public static boolean isWifi(Context ctx) {
		NetworkInfo info = ((ConnectivityManager) ctx.getSystemService(Context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
		if (info != null) {
			return info.getType() == ConnectivityManager.TYPE_WIFI;
		}
		return false;
	}

	public static int getStateDescriptionRId(State state) {
		switch(state) {
		case NONE : return R.string.st_none_descr;
		case INITIALISE : return R.string.st_initialise_descr;
		case ACTIVATE : return R.string.st_activate_descr;
		case ASSOCIATE : return R.string.st_assosiate_descr;
		case CHECK_CONFIG_VERSION : return R.string.st_check_config_version_descr;
		case DOWNLOAD_CONFIG : return R.string.st_download_config_descr;
		case EXECUTE_QUEUE : return R.string.st_execute_queue_descr;
		case RUN_INIT_TESTS : return R.string.st_run_init_test_descr;
		case SUBMIT_RESULTS : return R.string.st_submit_results_descr;
		case SHUTDOWN : return R.string.st_shutdown_descr;
		default : throw new RuntimeException("no such state: " + state);
		}
	}

	public static boolean isPhoneAssosiated(Context ctx) {
		String imei = PhoneIdentityDataCollector.getImei(ctx);
		for (DeviceDescription dd : SK2AppSettings.getInstance().getDevices()) {
			if (dd.isCurrentDevice(imei)) return true;
		}
		return false;
	}

	public static void removeDeviceForImei(String imei, List<DeviceDescription> list) {
		for (int i = 0; i < list.size(); i++) {
			DeviceDescription dd = list.get(i);
			if (dd.getMac().equals(imei)) {
				list.remove(i);
				return;
			}
		}
	}

	public static  String stringEncoding(String value){
		Pattern p = Pattern.compile("%u([a-zA-Z0-9]{4})");
		Matcher m = p.matcher(value);
		StringBuffer sb = new StringBuffer();
		while(m.find()){
			m.appendReplacement(sb, String.valueOf((char)Integer.parseInt(m.group(1), 16)));
		}
		m.appendTail(sb);
		return sb.toString();
	}

	public static boolean isThisDeviceAnEmulator() {
		if (Build.FINGERPRINT.startsWith("generic")) {
			// This is an Emulator!
			return true;
		}

		// This is a real device!
		return false;
	}

	// http://stackoverflow.com/questions/7085644/how-to-check-if-apk-is-signed-or-debug-build
	private static final X500Principal DEBUG_DN = new X500Principal("CN=Android Debug,O=Android,C=US");
	public static boolean isDebuggable(Context ctx)
	{
		boolean debuggable = false;

		try
		{
			PackageInfo pinfo = ctx.getPackageManager().getPackageInfo(ctx.getPackageName(),PackageManager.GET_SIGNATURES);
			Signature signatures[] = pinfo.signatures;

			CertificateFactory cf = CertificateFactory.getInstance("X.509");

			for ( int i = 0; i < signatures.length;i++)
			{   
				ByteArrayInputStream stream = new ByteArrayInputStream(signatures[i].toByteArray());
				X509Certificate cert = (X509Certificate) cf.generateCertificate(stream);       
				debuggable = cert.getSubjectX500Principal().equals(DEBUG_DN);
				if (debuggable)
					break;
			}
		}
		catch (NameNotFoundException e)
		{
			//debuggable variable will remain false
		}
		catch (CertificateException e)
		{
			//debuggable variable will remain false
		}
		return debuggable;
	}

}
