package com.samknows.tests;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Random;
import java.util.concurrent.atomic.AtomicBoolean;



import android.util.Log;


public class HttpSingleTest implements Runnable {

	
	private static final String TAG = HttpSingleTest.class.getSimpleName();

	/*
	 * error codes and contraints
	 */
	private static final int BYTESREADERR = -1;

	/*
	 * Time in microseconds
	 */
	private static long microTime() {
		return System.nanoTime() / 1000;
	}

	public HttpSingleTest(boolean downstream) {
		this.downstream = downstream;
	}

	private void setError(String error) {
		synchronized (this) {
			errorString = error;
		}
	}
	private String errorString = "";
	
	public boolean isReady() {
		if (target.length() == 0) {
			setError("Target empty");
			return false;
		}
		if (port == 0) {
			setError("Port is zero");
			return false;
		}
		if (warmupMaxTime == 0 && warmupMaxBytes == 0) {
			setError("No warmup parameter defined");
			return false;
		}
		if (transferMaxTime == 0 && transferMaxBytes == 0) {
			setError("No transfer parameter defined");
			return false;
		}
		if (!downstream && (sendDataChunkSize == 0 || postDataLength == 0)) {
			setError("Upload parameter missing");
			return false;
		}
		if (downstream && bufferSize == 0) {
			setError("Buffer size missing for download");
			return false;
		}
		return true;
	}
	
	public int getTransferBytes() {
		return transferBytes;
	}

	public int getWarmupBytes() {
		return warmupBytes;
	}
	
	public long getWarmupTime(){
		return warmupTime;
	}
	
	public long getStartWarmup(){
		return startWarmup;
	}
	
	public long getTransferTime(){
		return transferTime;
	}
	
//	private boolean setErrorIfEmpty(String error, Exception e) {
//		String exErr = e.getMessage() == null ? "No expetion message" : e
//				.getMessage();
//		return setErrorIfEmpty(error + " " + exErr);
//	}
	
	private boolean setErrorIfEmpty(String error) {
		boolean ret = false;
		synchronized (this) {
			if (errorString.equals("")) {
				errorString = error;
				ret = true;
			}
		}
		return ret;
	}
	

	public void download() {

		OutputStream connOut = null;
		InputStream connIn = null;
		byte[] buff = new byte[bufferSize];
		int readBytes = 0;
		HttpURLConnection con = null;
		String urlString = "http://"+target+":"+port+"/"+"104857600";	//100MByte
		
		
		try {
			URL url = new URL(urlString);
			con = (HttpURLConnection) url.openConnection();
			con.setRequestMethod("GET");
			con.setUseCaches(false);
			con.setDoOutput(false);	
			con.setDoInput(true);
			con.setConnectTimeout(5000);
			
			con.connect();
			//connIn = con.getInputStream();
			connIn = new BufferedInputStream(con.getInputStream(),2*1024*1024);
			int httpResponse = con.getResponseCode();
			if (httpResponse != HttpURLConnection.HTTP_OK) {
				setErrorIfEmpty("Http response received: " + httpResponse);
				error.set(true);
				Log.e(TAG, "Http response received: " + httpResponse);
			}
		} catch (MalformedURLException e) {
			error.set(true);
			Log.e(TAG,"MalformedURLException");
		} catch (IOException e) {
			error.set(true);
			Log.e(TAG,"IOException");
		}
		
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}
		// warmup, can be based on time constraint or data usage constraint
		do {
			try {
				readBytes = connIn.read(buff, 0, buff.length);
			} catch (IOException io) {
				readBytes = BYTESREADERR;
				error.set(true);
			}
		} while (!isWarmupDone(readBytes));
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}
		do {
			try {
				readBytes = connIn.read(buff, 0, buff.length);
			} catch (IOException io) {
				readBytes = BYTESREADERR;
			}
		} while (!isTransferDone(readBytes));

		testDone = true;
		waitForDisconnect();
		closeConnection(con, connIn, connOut);
//		mConn = con;
//		mConnIn = connIn;
//		mConnOut = connOut;
	}
	public void setTransferReady(long time){
		readyTransfer = time;
	}

	// the warmup ends also if there is a problem on any connection
	private synchronized boolean isWarmupDone(int bytes) {
		boolean ret = false;
		boolean timeWarmup = false;
		boolean bytesWarmup = false;
		// if there is an error the test must stop and report it
		if (bytes == BYTESREADERR) {
			setErrorIfEmpty("read error");
			bytes = 0; // do not modify the bytes counters
			error.set(true);
		}

		warmupBytes += bytes;
		if (startWarmup == 0) {
			startWarmup = microTime();
		}
		warmupTime = microTime() - startWarmup;
		
		// if warmup max time is set and time has exceeded its values set time
		// warmup to true
		timeWarmup = warmupMaxTime > 0 && warmupTime >= warmupMaxTime;

		// if warmup max bytes is set and bytes counter exceeded its value set
		// bytesWarmup to true
		bytesWarmup = warmupMaxBytes > 0 && warmupBytes >= warmupMaxBytes;

		// if a condition happened increment the warmupDoneCounter
		if (timeWarmup || bytesWarmup) {
			ret = true;
		}
		
		// if there is an error notify all, some thread might be waiting for
		// other to finish the warmup, and exit
		if (error.get()) {
			ret = true;
			if(m_listener != null) m_listener.noti(this);

		}// warmup is finished, wait for other threads if any
		else if (ret) {
			warmUpDone = true;
			readyTransfer = 0;
			Log.e(TAG,"warmup end! warmupBytes:"+warmupBytes+" warmupTime:"+warmupTime);
			if(m_listener != null) m_listener.noti(this);
			while(readyTransfer == 0){
				try {
					Thread.sleep(1);
				} catch (InterruptedException e) {
					error.set(true);
				}
			}
		}
		else {
			if(m_listener != null) m_listener.noti(this);
		}

		return ret;
	}
	


	private synchronized boolean isTransferDone(int bytes) {
		boolean ret = false;

		
		// In case an error occurred stop the test
		if (bytes == BYTESREADERR) {
			error.set(true);
			bytes = 0;
		}
		transferBytes += bytes;

		// if startTransfer is 0 this is the first call to isTransferDone
		if (startTransfer == 0) {
			startTransfer = microTime();
		}
		transferTime = microTime() - startTransfer;
		
		// if transfermax time is
		if ((transferMaxTime > 0) && (transferTime > transferMaxTime)) {
			ret = true;
		}
		if ((transferMaxBytes > 0) && (transferBytes > transferMaxBytes)) {
			ret = true;
		}
		if(ret){
			Log.e(TAG,"transfer_end transferBytes:"+transferBytes+" transferTime:"+transferTime);
		}
		
		if (transferBytes > 0) {
			testStatus = "OK";
		}

		if (error.get()) {
			ret = true;
		}
		if(m_listener != null) m_listener.noti(this);

		return ret;
	}
	
	@Override
	public void run() {

		if (downstream) {
			download();
		} else {
			upload();
		}
	}

	public void upload() {

		
		OutputStream connOut = null;
		InputStream connIn = null;
		Random generator = new Random();
		byte[] buff = new byte[sendDataChunkSize];
		HttpURLConnection con = null;
		String urlString = "http://"+target+":"+port;

		try {
			URL url = new URL(urlString);
			con = (HttpURLConnection) url.openConnection();
			con.setRequestMethod("POST");
			con.setUseCaches(false);
			con.setDoOutput(true);
			con.setChunkedStreamingMode(0);
//			con.setRequestProperty("Content-Length",""+postDataLength);
			con.setRequestProperty("Content-Type","application/x-www-form-urlencoded");
			con.setRequestProperty("Expect","100-continue");
			con.setConnectTimeout(5000);
		
			con.connect();
			connOut = con.getOutputStream();
		} catch (MalformedURLException e) {
			error.set(true);
		} catch (IOException e) {
			error.set(true);
		}

		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}

		do {
			if (randomEnabled) {
				generator.nextBytes(buff);
			}
			try {
				connOut.write(buff);
				connOut.flush();
			} catch (IOException ioe) {
				error.set(true);
			}
		} while (!isWarmupDone(sendDataChunkSize));
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}
		do {
			if (randomEnabled) {
				generator.nextBytes(buff);
			}
			try {
				connOut.write(buff);
				connOut.flush();
			} catch (IOException ioe) {
				error.set(true);
			}
		} while (!isTransferDone(sendDataChunkSize));
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}
		
		try {
			int httpResponse = con.getResponseCode();
			if (httpResponse != HttpURLConnection.HTTP_OK) {
				setErrorIfEmpty("Http response received: " + httpResponse);
				error.set(true);
			}
		} catch (IOException ioe) {
			error.set(true);
		}
		
		testDone = true;
		//waitForDisconnect();
		closeConnection(con, connIn, connOut);
//		mConn = con;
//		mConnIn = connIn;
//		mConnOut = connOut;
	}
	
	private synchronized void waitForDisconnect(){
		if(m_listener != null) {
			if(m_listener.isWaitForDisconnect(this)){
				try {
					Log.d(TAG,"wait thread for disconnect");
					wait();
				} catch (InterruptedException e) {
					
				}					
			}
		}
	}

	private void closeConnection(HttpURLConnection h, InputStream i, OutputStream o) {
		mConn = h;
		mConnIn = i;
		mConnOut = o;
		disconnect();
	}
	

	public void setTarget(String target) {
		this.target = target;
	}

	public void setPort(int port) {
		this.port = port;
	}

	public void setWarmupMaxTime(int time) {
		warmupMaxTime = time;
	}

	public void setWarmupMaxBytes(int bytes) {
		warmupMaxBytes = bytes;
	}

	public void setTransferMaxTime(int time) {
		transferMaxTime = time;
	}

	public void setTransferMaxBytes(int bytes) {
		transferMaxBytes = bytes;
	}

	public void setBufferSize(int size) {
		bufferSize = size;
	}

	public void setReceiveBufferSize(int size) {
		bufferSize = size;
	}

	public void setSendDataChunk(int size) {
		sendDataChunkSize = size;
	}

	public void setPostDataLenght(int l) {
		postDataLength = l;
	}

//	public boolean isProgressAvailable() {
//		boolean ret = false;
//		if (transferMaxTime > 0) {
//			ret = true;
//		} else if (transferMaxBytes > 0) {
//			ret = true;
//		}
//		return ret;
//	}
	
	public interface StateListener {
		public abstract void noti(HttpSingleTest test);
		public abstract boolean isWaitForDisconnect(HttpSingleTest test);
	}
	public void setStateListener(StateListener listener){
		m_listener = listener;
	}
	private StateListener m_listener = null;
	

	boolean upload = true;
	boolean randomEnabled = false;
	String ipAddress = "";
	public boolean warmUpDone = false;
	public boolean testDone = false;
	int postDataLength = 0;

	// warmup variables
	long startWarmup = 0;
	long warmupTime = 0;
	long warmupMaxTime = 0;
	int warmupBytes = 0;
	int warmupMaxBytes = 0;

	// transfer variables
	long readyTransfer = 0;		// during time 1sec
	long startTransfer = 0;
	long transferTime = 0;
	int transferBytes = 0;
	long transferMaxTime = 0;
	int transferMaxBytes = 0;

	// test variables
	int bufferSize = 0;
	int sendDataChunkSize = 0;
	String testStatus = "FAIL";

	// Connection variables
	String target = "";
	int port = 0;
	boolean downstream = true;

	// Test Status variable
	public AtomicBoolean error = new AtomicBoolean(false);
	
	
	
	private HttpURLConnection mConn;
	private OutputStream mConnOut;
	private InputStream mConnIn;
	synchronized public void disconnect(){
		Log.d(TAG,"start disconnect");
		long startTime = System.currentTimeMillis();
		if(mConnOut != null){
			try {
				mConnOut.close();
			} catch (IOException e) {
				Log.e(TAG,"HTTP OutputStrem error:"+e.getMessage());
			}
			mConnOut = null;
		}
		if(mConnIn != null){
			try {
//				int count = mConnIn.available();
//				mConnIn.skip(count);
				mConnIn.close();
				//Log.d(TAG,"finish close :"+(System.currentTimeMillis() - startTime)+"ms");
			} catch (IOException e) {
				Log.e(TAG,"HTTP InputStrem error:"+e.getMessage());
			}
			mConnIn = null;
		}
		if(mConn != null){
			mConn.disconnect();
			mConn = null;
		}
		Log.d(TAG,"finish disconnect :"+(System.currentTimeMillis() - startTime)+"ms");
	
	}
	public Thread runDisconnect(){
		Thread thread = new Thread(new Runnable(){
			@Override
			public void run() {
				disconnect();
			}
		});
		thread.start();
		return thread;
	}

}
